@@ -42,7 +42,7 @@
 
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/devicesupport/deviceprocessesdialog.h>
-#include <projectexplorer/devicesupport/deviceprocesslist.h>
+#include <projectexplorer/devicesupport/idevice.h>
 #include <projectexplorer/environmentaspect.h> // For the environment
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
@@ -73,8 +73,6 @@
 
 #include <qtsupport/qtkitinformation.h>
 
-#include <ssh/sshconnection.h>
-
 #include <QTcpServer>
 #include <QTimer>
 
@@ -95,117 +93,32 @@ DebuggerEngine *createQmlEngine();
 DebuggerEngine *createLldbEngine();
 DebuggerEngine *createUvscEngine();
 
-class LocalProcessRunner : public RunWorker
+static QString noEngineMessage()
 {
-    Q_DECLARE_TR_FUNCTIONS(Debugger::Internal::LocalProcessRunner)
+   return DebuggerPlugin::tr("Unable to create a debugging engine.");
+}
 
-public:
-    LocalProcessRunner(DebuggerRunTool *runTool, const CommandLine &command)
-        : RunWorker(runTool->runControl()), m_runTool(runTool), m_command(command)
-    {
-        connect(&m_proc, &QProcess::errorOccurred,
-                this, &LocalProcessRunner::handleError);
-        connect(&m_proc, &QProcess::readyReadStandardOutput,
-                this, &LocalProcessRunner::handleStandardOutput);
-        connect(&m_proc, &QProcess::readyReadStandardError,
-                this, &LocalProcessRunner::handleStandardError);
-        connect(&m_proc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-                this, &LocalProcessRunner::handleFinished);
-    }
-
-    void start() override
-    {
-        m_proc.setCommand(m_command);
-        m_proc.start();
-    }
-
-    void stop() override
-    {
-        m_proc.terminate();
-    }
-
-    void handleStandardOutput()
-    {
-        const QByteArray ba = m_proc.readAllStandardOutput();
-        const QString msg = QString::fromLocal8Bit(ba, ba.length());
-        m_runTool->appendMessage(msg, StdOutFormat);
-    }
-
-    void handleStandardError()
-    {
-        const QByteArray ba = m_proc.readAllStandardError();
-        const QString msg = QString::fromLocal8Bit(ba, ba.length());
-        m_runTool->appendMessage(msg, StdErrFormat);
-    }
-
-    void handleFinished()
-    {
-        if (m_proc.exitStatus() == QProcess::NormalExit && m_proc.exitCode() == 0) {
-            // all good.
-            reportDone();
-        } else {
-            reportFailure(tr("Upload failed: %1").arg(m_proc.errorString()));
-        }
-    }
-
-    void handleError(QProcess::ProcessError error)
-    {
-        QString msg;
-        switch (error) {
-        case QProcess::FailedToStart:
-            msg = tr("The upload process failed to start. Shell missing?");
-            break;
-        case QProcess::Crashed:
-            msg = tr("The upload process crashed some time after starting "
-                     "successfully.");
-            break;
-        case QProcess::Timedout:
-            msg = tr("The last waitFor...() function timed out. "
-                     "The state of QProcess is unchanged, and you can try calling "
-                     "waitFor...() again.");
-            break;
-        case QProcess::WriteError:
-            msg = tr("An error occurred when attempting to write "
-                     "to the upload process. For example, the process may not be running, "
-                     "or it may have closed its input channel.");
-            break;
-        case QProcess::ReadError:
-            msg = tr("An error occurred when attempting to read from "
-                     "the upload process. For example, the process may not be running.");
-            break;
-        default:
-            msg = tr("An unknown error in the upload process occurred. "
-                     "This is the default return value of error().");
-        }
-
-        m_runTool->showMessage(msg, StatusBar);
-        Core::AsynchronousMessageBox::critical(tr("Error"), msg);
-    }
-
-    QPointer<DebuggerRunTool> m_runTool;
-    CommandLine m_command;
-    Utils::QtcProcess m_proc;
-};
+static QString noDebuggerInKitMessage()
+{
+   return DebuggerPlugin::tr("The kit does not have a debugger set.");
+}
 
 class CoreUnpacker final : public RunWorker
 {
 public:
-    CoreUnpacker(RunControl *runControl, const QString &coreFileName)
-        : RunWorker(runControl), m_coreFileName(coreFileName)
+    CoreUnpacker(RunControl *runControl, const FilePath &coreFilePath)
+        : RunWorker(runControl), m_coreFilePath(coreFilePath)
     {}
 
-    QString coreFileName() const { return m_tempCoreFileName; }
+    FilePath coreFileName() const { return m_tempCoreFilePath; }
 
 private:
     ~CoreUnpacker() final
     {
-        m_coreUnpackProcess.blockSignals(true);
-        m_coreUnpackProcess.terminate();
-        m_coreUnpackProcess.deleteLater();
         if (m_tempCoreFile.isOpen())
             m_tempCoreFile.close();
 
-        QFile::remove(m_tempCoreFileName);
+        m_tempCoreFilePath.removeFile();
     }
 
     void start() final
@@ -213,40 +126,50 @@ private:
         {
             Utils::TemporaryFile tmp("tmpcore-XXXXXX");
             tmp.open();
-            m_tempCoreFileName = tmp.fileName();
+            m_tempCoreFilePath = FilePath::fromString(tmp.fileName());
         }
 
-        m_coreUnpackProcess.setWorkingDirectory(TemporaryDirectory::masterDirectoryPath());
-        connect(&m_coreUnpackProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-                this, &CoreUnpacker::reportStarted);
+        m_coreUnpackProcess.setWorkingDirectory(TemporaryDirectory::masterDirectoryFilePath());
+        connect(&m_coreUnpackProcess, &QtcProcess::done, this, [this] {
+            if (m_coreUnpackProcess.error() == QProcess::UnknownError) {
+                reportStopped();
+                return;
+            }
+            reportFailure("Error unpacking " + m_coreFilePath.toUserOutput());
+        });
 
         const QString msg = DebuggerRunTool::tr("Unpacking core file to %1");
-        appendMessage(msg.arg(m_tempCoreFileName), LogMessageFormat);
+        appendMessage(msg.arg(m_tempCoreFilePath.toUserOutput()), LogMessageFormat);
 
-        if (m_coreFileName.endsWith(".lzo")) {
-            m_coreUnpackProcess.start("lzop", {"-o", m_tempCoreFileName, "-x", m_coreFileName});
+        if (m_coreFilePath.endsWith(".lzo")) {
+            m_coreUnpackProcess.setCommand({"lzop", {"-o", m_tempCoreFilePath.path(),
+                                                     "-x", m_coreFilePath.path()}});
+            reportStarted();
+            m_coreUnpackProcess.start();
             return;
         }
 
-        if (m_coreFileName.endsWith(".gz")) {
-            appendMessage(msg.arg(m_tempCoreFileName), LogMessageFormat);
-            m_tempCoreFile.setFileName(m_tempCoreFileName);
+        if (m_coreFilePath.endsWith(".gz")) {
+            appendMessage(msg.arg(m_tempCoreFilePath.toUserOutput()), LogMessageFormat);
+            m_tempCoreFile.setFileName(m_tempCoreFilePath.path());
             m_tempCoreFile.open(QFile::WriteOnly);
-            connect(&m_coreUnpackProcess, &QProcess::readyRead, this, [this] {
-                m_tempCoreFile.write(m_coreUnpackProcess.readAll());
+            connect(&m_coreUnpackProcess, &QtcProcess::readyReadStandardOutput, this, [this] {
+                m_tempCoreFile.write(m_coreUnpackProcess.readAllStandardOutput());
             });
-            m_coreUnpackProcess.start("gzip", {"-c", "-d", m_coreFileName});
+            m_coreUnpackProcess.setCommand({"gzip", {"-c", "-d", m_coreFilePath.path()}});
+            reportStarted();
+            m_coreUnpackProcess.start();
             return;
         }
 
         QTC_CHECK(false);
-        reportFailure("Unknown file extension in " + m_coreFileName);
+        reportFailure("Unknown file extension in " + m_coreFilePath.toUserOutput());
     }
 
     QFile m_tempCoreFile;
-    QString m_coreFileName;
-    QString m_tempCoreFileName;
-    QProcess m_coreUnpackProcess;
+    FilePath m_coreFilePath;
+    FilePath m_tempCoreFilePath;
+    QtcProcess m_coreUnpackProcess;
 };
 
 class DebuggerRunToolPrivate
@@ -288,7 +211,7 @@ void DebuggerRunTool::setStartMode(DebuggerStartMode startMode)
             projects.removeOne(startupProject);
             projects.insert(0, startupProject);
         }
-        foreach (Project *project, projects)
+        for (Project *project : qAsConst(projects))
             m_runParameters.projectSourceFiles.append(project->files(Project::SourceFiles));
         if (!projects.isEmpty())
             m_runParameters.projectSourceDirectory = projects.first()->projectDirectory();
@@ -391,10 +314,11 @@ void DebuggerRunTool::setUseTerminal(bool on)
     bool useCdbConsole = m_runParameters.cppEngineType == CdbEngineType
             && (m_runParameters.startMode == StartInternal
                 || m_runParameters.startMode == StartExternal)
-            && boolSetting(UseCdbConsole);
+            && debuggerSettings()->useCdbConsole.value();
 
     if (on && !d->terminalRunner && !useCdbConsole) {
-        d->terminalRunner = new TerminalRunner(runControl(), m_runParameters.inferior);
+        d->terminalRunner =
+            new TerminalRunner(runControl(), [this] { return m_runParameters.inferior; });
         addStartDependency(d->terminalRunner);
     }
     if (!on && d->terminalRunner) {
@@ -402,6 +326,11 @@ void DebuggerRunTool::setUseTerminal(bool on)
     }
 }
 
+void DebuggerRunTool::setRunAsRoot(bool on)
+{
+    m_runParameters.runAsRoot = on;
+}
+
 void DebuggerRunTool::setCommandsAfterConnect(const QString &commands)
 {
     m_runParameters.commandsAfterConnect = commands;
@@ -411,18 +340,7 @@ void DebuggerRunTool::setCommandsForReset(const QString &commands)
 {
     m_runParameters.commandsForReset = commands;
 }
-
-void DebuggerRunTool::setServerStartScript(const FilePath &serverStartScript)
-{
-    if (!serverStartScript.isEmpty()) {
-        // Provide script information about the environment
-        const CommandLine serverStarter(serverStartScript,
-            {m_runParameters.inferior.executable.toString(), m_runParameters.remoteChannel});
-        addStartDependency(new LocalProcessRunner(this, serverStarter));
-    }
-}
-
-void DebuggerRunTool::setDebugInfoLocation(const QString &debugInfoLocation)
+ void DebuggerRunTool::setDebugInfoLocation(const FilePath &debugInfoLocation)
 {
     m_runParameters.debugInfoLocation = debugInfoLocation;
 }
@@ -452,7 +370,7 @@ void DebuggerRunTool::setTestCase(int testCase)
     m_runParameters.testCase = testCase;
 }
 
-void DebuggerRunTool::setOverrideStartScript(const QString &script)
+void DebuggerRunTool::setOverrideStartScript(const FilePath &script)
 {
     m_runParameters.overrideStartScript = script;
 }
@@ -469,7 +387,7 @@ void DebuggerRunTool::setInferior(const Runnable &runnable)
 
 void DebuggerRunTool::setInferiorExecutable(const FilePath &executable)
 {
-    m_runParameters.inferior.executable = executable;
+    m_runParameters.inferior.command.setExecutable(executable);
 }
 
 void DebuggerRunTool::setInferiorEnvironment(const Utils::Environment &env)
@@ -477,11 +395,6 @@ void DebuggerRunTool::setInferiorEnvironment(const Utils::Environment &env)
     m_runParameters.inferior.environment = env;
 }
 
-void DebuggerRunTool::setInferiorDevice(IDevice::ConstPtr device)
-{
-    m_runParameters.inferior.device = device;
-}
-
 void DebuggerRunTool::setRunControlName(const QString &name)
 {
     m_runParameters.displayName = name;
@@ -492,7 +405,7 @@ void DebuggerRunTool::setStartMessage(const QString &msg)
     m_runParameters.startMessage = msg;
 }
 
-void DebuggerRunTool::setCoreFileName(const QString &coreFile, bool isSnapshot)
+void DebuggerRunTool::setCoreFilePath(const FilePath &coreFile, bool isSnapshot)
 {
     if (coreFile.endsWith(".gz") || coreFile.endsWith(".lzo")) {
         d->coreUnpacker = new CoreUnpacker(runControl(), coreFile);
@@ -544,17 +457,17 @@ void DebuggerRunTool::start()
             QTC_ASSERT(qmlServerPort > 0, reportFailure(); return);
             QString mode = QString("port:%1").arg(qmlServerPort);
 
-            CommandLine cmd{m_runParameters.inferior.executable};
+            CommandLine cmd{m_runParameters.inferior.command.executable()};
             cmd.addArg(qmlDebugCommandLineArguments(QmlDebug::QmlDebuggerServices, mode, true));
-            cmd.addArgs(m_runParameters.inferior.commandLineArguments, CommandLine::Raw);
+            cmd.addArgs(m_runParameters.inferior.command.arguments(), CommandLine::Raw);
 
-            m_runParameters.inferior.setCommandLine(cmd);
+            m_runParameters.inferior.command = cmd;
         }
     }
 
     // User canceled input dialog asking for executable when working on library project.
     if (m_runParameters.startMode == StartInternal
-            && m_runParameters.inferior.executable.isEmpty()
+            && m_runParameters.inferior.command.isEmpty()
             && m_runParameters.interpreter.isEmpty()) {
         reportFailure(tr("No executable specified."));
         return;
@@ -577,19 +490,19 @@ void DebuggerRunTool::start()
         return;
 
     if (m_runParameters.cppEngineType == CdbEngineType
-            && Utils::is64BitWindowsBinary(m_runParameters.inferior.executable.toString())
-            && !Utils::is64BitWindowsBinary(m_runParameters.debugger.executable.toString())) {
+            && Utils::is64BitWindowsBinary(m_runParameters.inferior.command.executable())
+            && !Utils::is64BitWindowsBinary(m_runParameters.debugger.command.executable())) {
         reportFailure(
             DebuggerPlugin::tr(
                 "%1 is a 64 bit executable which can not be debugged by a 32 bit Debugger.\n"
                 "Please select a 64 bit Debugger in the kit settings for this kit.")
-                .arg(m_runParameters.inferior.executable.toUserOutput()));
+                .arg(m_runParameters.inferior.command.executable().toUserOutput()));
         return;
     }
 
     Utils::globalMacroExpander()->registerFileVariables(
                 "DebuggedExecutable", tr("Debugged executable"),
-                [this] { return m_runParameters.inferior.executable.toString(); }
+                [this] { return m_runParameters.inferior.command.executable(); }
     );
 
     runControl()->setDisplayName(m_runParameters.displayName);
@@ -619,8 +532,8 @@ void DebuggerRunTool::start()
                 break;
             default:
                 if (!m_runParameters.isQmlDebugging) {
-                    reportFailure(DebuggerPlugin::tr("Unable to create a debugging engine. "
-                                                     "Please select a Debugger Setting from the Run page of the project mode."));
+                    reportFailure(noEngineMessage() + '\n' +
+                        DebuggerPlugin::tr("Specify Debugger settings in Projects > Run."));
                     return;
                 }
                 // Can happen for pure Qml.
@@ -638,7 +551,10 @@ void DebuggerRunTool::start()
     }
 
     if (!m_engine) {
-        reportFailure(DebuggerPlugin::tr("Unable to create a debugging engine."));
+        QString msg = noEngineMessage();
+        if (!DebuggerKitAspect::debugger(runControl()->kit()))
+            msg += '\n' + noDebuggerInKitMessage();
+        reportFailure(msg);
         return;
     }
 
@@ -660,15 +576,13 @@ void DebuggerRunTool::start()
     ++d->engineStopsNeeded;
 
     connect(m_engine, &DebuggerEngine::attachToCoreRequested, this, [this](const QString &coreFile) {
-        auto runConfig = runControl()->runConfiguration();
-        QTC_ASSERT(runConfig, return);
         auto rc = new RunControl(ProjectExplorer::Constants::DEBUG_RUN_MODE);
-        rc->setRunConfiguration(runConfig);
+        rc->copyDataFromRunControl(runControl());
         auto name = QString(tr("%1 - Snapshot %2").arg(runControl()->displayName()).arg(++d->snapshotCounter));
         auto debugger = new DebuggerRunTool(rc);
-        debugger->setStartMode(AttachCore);
+        debugger->setStartMode(AttachToCore);
         debugger->setRunControlName(name);
-        debugger->setCoreFileName(coreFile, true);
+        debugger->setCoreFilePath(FilePath::fromString(coreFile), true);
         debugger->startRunControl();
     });
 
@@ -694,17 +608,30 @@ void DebuggerRunTool::start()
 
     if (m_runParameters.startMode == StartInternal) {
         QStringList unhandledIds;
-//        for (const GlobalBreakpoint &bp : BreakpointManager::globalBreakpoints()) {
-//            if (bp->isEnabled() && !m_engine->acceptsBreakpoint(bp))
-//                unhandledIds.append(bp.id().toString());
-//        }
+        bool hasQmlBreakpoints = false;
+        for (const GlobalBreakpoint &gbp : BreakpointManager::globalBreakpoints()) {
+            if (gbp->isEnabled()) {
+                const BreakpointParameters &bp = gbp->requestedParameters();
+                hasQmlBreakpoints = hasQmlBreakpoints || bp.isQmlFileAndLineBreakpoint();
+                if (!m_engine->acceptsBreakpoint(bp)) {
+                    if (!m_engine2 || !m_engine2->acceptsBreakpoint(bp))
+                        unhandledIds.append(gbp->displayName());
+                }
+            }
+        }
         if (!unhandledIds.isEmpty()) {
             QString warningMessage =
                     DebuggerPlugin::tr("Some breakpoints cannot be handled by the debugger "
-                                       "languages currently active, and will be ignored.\n"
+                                       "languages currently active, and will be ignored.<p>"
                                        "Affected are breakpoints %1")
                     .arg(unhandledIds.join(", "));
 
+            if (hasQmlBreakpoints) {
+                warningMessage += "<p>" +
+                    DebuggerPlugin::tr("QML debugging needs to be enabled both in the Build "
+                                       "and the Run settings.");
+            }
+
             showMessage(warningMessage, LogWarning);
 
             static bool checked = true;
@@ -717,7 +644,8 @@ void DebuggerRunTool::start()
         }
     }
 
-    appendMessage(tr("Debugging starts"), NormalMessageFormat);
+    appendMessage(tr("Debugging %1 ...").arg(m_runParameters.inferior.command.toUserOutput()),
+                  NormalMessageFormat);
     QString debuggerName = m_engine->objectName();
     if (m_engine2)
         debuggerName += ' ' + m_engine2->objectName();
@@ -761,7 +689,12 @@ void DebuggerRunTool::handleEngineFinished(DebuggerEngine *engine)
 {
     engine->prepareForRestart();
     if (--d->engineStopsNeeded == 0) {
-        appendMessage(tr("Debugging has finished"), NormalMessageFormat);
+        QString cmd = m_runParameters.inferior.command.toUserOutput();
+        QString msg = engine->runParameters().exitCode // Main engine.
+            ? tr("Debugging of %1 has finished with exit code %2.")
+                .arg(cmd).arg(engine->runParameters().exitCode.value())
+            : tr("Debugging of %1 has finished.").arg(cmd);
+        appendMessage(msg, NormalMessageFormat);
         reportStopped();
     }
 }
@@ -804,7 +737,7 @@ bool DebuggerRunTool::fixupParameters()
 {
     DebuggerRunParameters &rp = m_runParameters;
     if (rp.symbolFile.isEmpty())
-        rp.symbolFile = rp.inferior.executable;
+        rp.symbolFile = rp.inferior.command.executable();
 
     // Copy over DYLD_IMAGE_SUFFIX etc
     for (const auto &var :
@@ -813,7 +746,7 @@ bool DebuggerRunTool::fixupParameters()
             rp.debugger.environment.set(var, rp.inferior.environment.expandedValueForKey(var));
 
     // validate debugger if C++ debugging is enabled
-    if (rp.isCppDebugging() && !rp.validationErrors.isEmpty()) {
+    if (!rp.validationErrors.isEmpty()) {
         reportFailure(rp.validationErrors.join('\n'));
         return false;
     }
@@ -835,12 +768,12 @@ bool DebuggerRunTool::fixupParameters()
         }
     }
 
-    if (!boolSetting(AutoEnrichParameters)) {
-        const QString sysroot = rp.sysRoot.toString();
+    if (!debuggerSettings()->autoEnrichParameters.value()) {
+        const FilePath sysroot = rp.sysRoot;
         if (rp.debugInfoLocation.isEmpty())
-            rp.debugInfoLocation = sysroot + "/usr/lib/debug";
+            rp.debugInfoLocation = sysroot / "/usr/lib/debug";
         if (rp.debugSourceLocation.isEmpty()) {
-            QString base = sysroot + "/usr/src/debug/";
+            QString base = sysroot.toString() + "/usr/src/debug/";
             rp.debugSourceLocation.append(base + "qt5base/src/corelib");
             rp.debugSourceLocation.append(base + "qt5base/src/gui");
             rp.debugSourceLocation.append(base + "qt5base/src/network");
@@ -858,11 +791,11 @@ bool DebuggerRunTool::fixupParameters()
         } else {
             service = QmlDebug::QmlDebuggerServices;
         }
-        if (rp.startMode != AttachExternal && rp.startMode != AttachCrashedExternal) {
+        if (rp.startMode != AttachToLocalProcess && rp.startMode != AttachToCrashedProcess) {
             QString qmlarg = rp.isCppDebugging() && rp.nativeMixedEnabled
                     ? QmlDebug::qmlDebugNativeArguments(service, false)
                     : QmlDebug::qmlDebugTcpArguments(service, rp.qmlServer);
-            QtcProcess::addArg(&rp.inferior.commandLineArguments, qmlarg);
+            rp.inferior.command.addArg(qmlarg);
         }
     }
 
@@ -875,12 +808,12 @@ bool DebuggerRunTool::fixupParameters()
     }
 
     if (HostOsInfo::isWindowsHost()) {
-        QtcProcess::SplitError perr;
-        rp.inferior.commandLineArguments =
-                QtcProcess::prepareArgs(rp.inferior.commandLineArguments, &perr,
-                                        HostOsInfo::hostOs(), nullptr,
-                                        &rp.inferior.workingDirectory).toWindowsArgs();
-        if (perr != QtcProcess::SplitOk) {
+        ProcessArgs::SplitError perr;
+        rp.inferior.command.setArguments(
+                ProcessArgs::prepareArgs(rp.inferior.command.arguments(), &perr,
+                                         HostOsInfo::hostOs(), nullptr,
+                                         &rp.inferior.workingDirectory).toWindowsArgs());
+        if (perr != ProcessArgs::SplitOk) {
             // perr == BadQuoting is never returned on Windows
             // FIXME? QTCREATORBUG-2809
             reportFailure(DebuggerPlugin::tr("Debugging complex command lines "
@@ -892,6 +825,9 @@ bool DebuggerRunTool::fixupParameters()
     if (rp.isNativeMixedDebugging())
         rp.inferior.environment.set("QV4_FORCE_INTERPRETER", "1");
 
+    if (debuggerSettings()->forceLoggingToConsole.value())
+        rp.inferior.environment.set("QT_LOGGING_TO_CONSOLE", "1");
+
     return true;
 }
 
@@ -932,38 +868,56 @@ DebuggerRunTool::DebuggerRunTool(RunControl *runControl, AllowTerminal allowTerm
     m_runParameters.displayName = runControl->displayName();
 
     if (auto symbolsAspect = runControl->aspect<SymbolFileAspect>())
-        m_runParameters.symbolFile = symbolsAspect->filePath();
+        m_runParameters.symbolFile = symbolsAspect->filePath;
     if (auto terminalAspect = runControl->aspect<TerminalAspect>())
-        m_runParameters.useTerminal = terminalAspect->useTerminal();
+        m_runParameters.useTerminal = terminalAspect->useTerminal;
+    if (auto runAsRootAspect = runControl->aspect<RunAsRootAspect>())
+        m_runParameters.runAsRoot = runAsRootAspect->value;
 
     Kit *kit = runControl->kit();
     QTC_ASSERT(kit, return);
 
     m_runParameters.sysRoot = SysRootKitAspect::sysRoot(kit);
-    m_runParameters.macroExpander = kit->macroExpander();
+    m_runParameters.macroExpander = runControl->macroExpander();
     m_runParameters.debugger = DebuggerKitAspect::runnable(kit);
     m_runParameters.cppEngineType = DebuggerKitAspect::engineType(kit);
 
-    if (QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitAspect::qtVersion(kit))
+    if (QtSupport::QtVersion *qtVersion = QtSupport::QtKitAspect::qtVersion(kit))
         m_runParameters.qtPackageSourceLocation = qtVersion->qtPackageSourcePath().toString();
 
     if (auto aspect = runControl->aspect<DebuggerRunConfigurationAspect>()) {
-        if (!aspect->useCppDebugger())
+        if (!aspect->useCppDebugger)
             m_runParameters.cppEngineType = NoEngineType;
-        m_runParameters.isQmlDebugging = aspect->useQmlDebugger();
-        m_runParameters.multiProcess = aspect->useMultiProcess();
-        m_runParameters.additionalStartupCommands = aspect->overrideStartup();
+        m_runParameters.isQmlDebugging = aspect->useQmlDebugger;
+        m_runParameters.multiProcess = aspect->useMultiProcess;
+        m_runParameters.additionalStartupCommands = aspect->overrideStartup;
+
+        if (aspect->useCppDebugger) {
+            if (DebuggerKitAspect::debugger(kit)) {
+                const Tasks tasks = DebuggerKitAspect::validateDebugger(kit);
+                for (const Task &t : tasks) {
+                    if (t.type != Task::Warning)
+                        m_runParameters.validationErrors.append(t.description());
+                }
+            } else {
+                m_runParameters.validationErrors.append(noDebuggerInKitMessage());
+            }
+        }
     }
 
-    m_runParameters.inferior = runnable();
+    Runnable inferior = runControl->runnable();
+    const FilePath &debuggerExecutable = m_runParameters.debugger.command.executable();
+    inferior.command.setExecutable(inferior.command.executable().onDevice(debuggerExecutable));
+    inferior.workingDirectory = inferior.workingDirectory.onDevice(debuggerExecutable);
     // Normalize to work around QTBUG-17529 (QtDeclarative fails with 'File name case mismatch'...)
-    m_runParameters.inferior.workingDirectory =
-            FileUtils::normalizePathName(m_runParameters.inferior.workingDirectory);
+    inferior.workingDirectory = inferior.workingDirectory.normalizedPathName();
+    m_runParameters.inferior = inferior;
+
     setUseTerminal(allowTerminal == DoAllowTerminal && m_runParameters.useTerminal);
 
     const QByteArray envBinary = qgetenv("QTC_DEBUGGER_PATH");
     if (!envBinary.isEmpty())
-        m_runParameters.debugger.executable = FilePath::fromString(QString::fromLocal8Bit(envBinary));
+        m_runParameters.debugger.command.setExecutable(FilePath::fromString(QString::fromLocal8Bit(envBinary)));
 
     if (Project *project = runControl->project()) {
         m_runParameters.projectSourceDirectory = project->projectDirectory();
@@ -977,29 +931,27 @@ DebuggerRunTool::DebuggerRunTool(RunControl *runControl, AllowTerminal allowTerm
     if (ok)
         m_runParameters.nativeMixedEnabled = bool(nativeMixedOverride);
 
-    // This will only be shown in some cases, but we don't want to access
-    // the kit at that time anymore.
-    const Tasks tasks = DebuggerKitAspect::validateDebugger(kit);
-    for (const Task &t : tasks) {
-        if (t.type != Task::Warning)
-            m_runParameters.validationErrors.append(t.description());
+
+    if (auto interpreterAspect = runControl->aspect<InterpreterAspect>()) {
+        if (auto mainScriptAspect = runControl->aspect<MainScriptAspect>()) {
+            const FilePath mainScript = mainScriptAspect->filePath;
+            const FilePath interpreter = interpreterAspect->interpreter.command;
+            if (!interpreter.isEmpty() && mainScript.endsWith(".py")) {
+                m_runParameters.mainScript = mainScript;
+                m_runParameters.interpreter = interpreter;
+                if (auto args = runControl->aspect<ArgumentsAspect>())
+                    m_runParameters.inferior.command.addArgs(args->arguments, CommandLine::Raw);
+                m_engine = createPdbEngine();
+            }
+        }
     }
 
-    RunConfiguration *runConfig = runControl->runConfiguration();
-    if (runConfig && runConfig->property("supportsDebugger").toBool()) {
-        const QString mainScript = runConfig->property("mainScript").toString();
-        const QString interpreter = runConfig->property("interpreter").toString();
-        if (!interpreter.isEmpty() && mainScript.endsWith(".py")) {
-            m_runParameters.mainScript = mainScript;
-            m_runParameters.interpreter = interpreter;
-            const QString args = runConfig->property("arguments").toString();
-            if (!args.isEmpty()) {
-                if (!m_runParameters.inferior.commandLineArguments.isEmpty())
-                    m_runParameters.inferior.commandLineArguments.append(' ');
-                m_runParameters.inferior.commandLineArguments.append(args);
-            }
-            m_engine = createPdbEngine();
-        }
+    m_runParameters.dumperPath = Core::ICore::resourcePath("debugger/");
+    if (QtSupport::QtVersion *baseQtVersion = QtSupport::QtKitAspect::qtVersion(kit)) {
+        QtSupport::QtVersionNumber qtVersion = baseQtVersion->qtVersion();
+        m_runParameters.fallbackQtVersion = 0x10000 * int(qtVersion.majorVersion)
+                                            + 0x100 * int(qtVersion.minorVersion)
+                                            + int(qtVersion.patchVersion);
     }
 }
 
@@ -1018,7 +970,7 @@ void DebuggerRunTool::addSolibSearchDir(const QString &str)
 DebuggerRunTool::~DebuggerRunTool()
 {
     if (m_runParameters.isSnapshot && !m_runParameters.coreFile.isEmpty())
-        QFile::remove(m_runParameters.coreFile);
+        m_runParameters.coreFile.removeFile();
 
     delete m_engine2;
     m_engine2 = nullptr;
@@ -1085,35 +1037,31 @@ DebugServerRunner::DebugServerRunner(RunControl *runControl, DebugServerPortsGat
    : SimpleTargetRunner(runControl)
 {
     setId("DebugServerRunner");
-    const Runnable mainRunnable = runControl->runnable();
     addStartDependency(portsGatherer);
 
     QTC_ASSERT(portsGatherer, reportFailure(); return);
 
-    setStarter([this, runControl, mainRunnable, portsGatherer] {
+    setStartModifier([this, runControl, portsGatherer] {
         QTC_ASSERT(portsGatherer, reportFailure(); return);
 
-        Runnable debugServer;
-        debugServer.environment = mainRunnable.environment;
-        debugServer.workingDirectory = mainRunnable.workingDirectory;
-
-        QStringList args = QtcProcess::splitArgs(mainRunnable.commandLineArguments, OsTypeLinux);
-
         const bool isQmlDebugging = portsGatherer->useQmlServer();
         const bool isCppDebugging = portsGatherer->useGdbServer();
 
+        CommandLine cmd;
+
+        QStringList args = ProcessArgs::splitArgs(commandLine().arguments(), OsTypeLinux);
         if (isQmlDebugging) {
             args.prepend(QmlDebug::qmlDebugTcpArguments(QmlDebug::QmlDebuggerServices,
                                                         portsGatherer->qmlServer()));
         }
         if (isQmlDebugging && !isCppDebugging) {
-            debugServer.executable = mainRunnable.executable; // FIXME: Case should not happen?
+            cmd.setExecutable(commandLine().executable()); // FIXME: Case should not happen?
         } else {
-            debugServer.executable = FilePath::fromString(runControl->device()->debugServerPath());
-            if (debugServer.executable.isEmpty())
-                debugServer.executable = FilePath::fromString("gdbserver");
+            cmd.setExecutable(runControl->device()->debugServerPath());
+            if (cmd.isEmpty())
+                cmd.setExecutable(runControl->device()->filePath("gdbserver"));
             args.clear();
-            if (debugServer.executable.toString().contains("lldb-server")) {
+            if (cmd.executable().toString().contains("lldb-server")) {
                 args.append("platform");
                 args.append("--listen");
                 args.append(QString("*:%1").arg(portsGatherer->gdbServer().port()));
@@ -1129,9 +1077,9 @@ DebugServerRunner::DebugServerRunner(RunControl *runControl, DebugServerPortsGat
                     args.append(QString::number(m_pid.pid()));
             }
         }
-        debugServer.commandLineArguments = QtcProcess::joinArgs(args, OsTypeLinux);
+        cmd.setArguments(ProcessArgs::joinArgs(args, OsTypeLinux));
 
-        doStart(debugServer, runControl->device());
+        setCommandLine(cmd);
     });
 }
 
