@@ -81,45 +81,29 @@ bool AssetsLibraryWidget::eventFilter(QObject *obj, QEvent *event)
         if (obj == m_assetsWidget.data())
             QMetaObject::invokeMethod(m_assetsWidget->rootObject(), "handleViewFocusOut");
     } else if (event->type() == QMouseEvent::MouseMove) {
-        if (!m_assetsToDrag.isEmpty()) {
+        if (!m_assetsToDrag.isEmpty() && !m_model.isNull()) {
             QMouseEvent *me = static_cast<QMouseEvent *>(event);
             if ((me->globalPos() - m_dragStartPoint).manhattanLength() > 10) {
-                auto drag = new QDrag(this);
-                drag->setPixmap(m_assetsIconProvider->requestPixmap(m_assetsToDrag[0], nullptr, {128, 128}));
                 QMimeData *mimeData = new QMimeData;
-                mimeData->setData("application/vnd.bauhaus.libraryresource", m_assetsToDrag.join(',').toUtf8());
-                drag->setMimeData(mimeData);
-                drag->exec();
-                drag->deleteLater();
-
+                mimeData->setData(Constants::MIME_TYPE_ASSETS, m_assetsToDrag.join(',').toUtf8());
+                m_model->startDrag(mimeData,
+                                   m_assetsIconProvider->requestPixmap(m_assetsToDrag[0], nullptr, {128, 128}));
                 m_assetsToDrag.clear();
             }
         }
-    } else if (event->type() == QMouseEvent::MouseButtonRelease) {
-        m_assetsToDrag.clear();
-        QWidget *view = QmlDesignerPlugin::instance()->viewManager().widget("Navigator");
-        if (view) {
-            NavigatorWidget *navView = qobject_cast<NavigatorWidget *>(view);
-            if (navView) {
-                navView->setDragType("");
-                navView->update();
-            }
-        }
     }
 
     return QObject::eventFilter(obj, event);
 }
 
-AssetsLibraryWidget::AssetsLibraryWidget(AsynchronousImageCache &imageCache,
-                                     AsynchronousImageCache &asynchronousFontImageCache,
-                                     SynchronousImageCache &synchronousFontImageCache)
+AssetsLibraryWidget::AssetsLibraryWidget(AsynchronousImageCache &asynchronousFontImageCache,
+                                         SynchronousImageCache &synchronousFontImageCache)
     : m_itemIconSize(24, 24)
     , m_fontImageCache(synchronousFontImageCache)
     , m_assetsIconProvider(new AssetsLibraryIconProvider(synchronousFontImageCache))
     , m_fileSystemWatcher(new Utils::FileSystemWatcher(this))
-    , m_assetsModel(new AssetsLibraryModel(synchronousFontImageCache, m_fileSystemWatcher, this))
+    , m_assetsModel(new AssetsLibraryModel(m_fileSystemWatcher, this))
     , m_assetsWidget(new QQuickWidget(this))
-    , m_imageCache{imageCache}
 {
     m_assetCompressionTimer.setInterval(200);
     m_assetCompressionTimer.setSingleShot(true);
@@ -130,11 +114,13 @@ AssetsLibraryWidget::AssetsLibraryWidget(AsynchronousImageCache &imageCache,
     m_assetsWidget->installEventFilter(this);
 
     m_fontPreviewTooltipBackend = std::make_unique<PreviewTooltipBackend>(asynchronousFontImageCache);
+    // We want font images to have custom size, so don't scale them in the tooltip
+    m_fontPreviewTooltipBackend->setScaleImage(false);
     // Note: Though the text specified here appears in UI, it shouldn't be translated, as it's
     // a commonly used sentence to preview the font glyphs in latin fonts.
     // For fonts that do not have latin glyphs, the font family name will have to suffice for preview.
     m_fontPreviewTooltipBackend->setAuxiliaryData(
-        ImageCache::FontCollectorSizeAuxiliaryData{QSize{300, 300},
+        ImageCache::FontCollectorSizeAuxiliaryData{QSize{300, 150},
                                                    Theme::getColor(Theme::DStextColor).name(),
                                                    QStringLiteral("The quick brown fox jumps\n"
                                                                   "over the lazy dog\n"
@@ -188,6 +174,8 @@ AssetsLibraryWidget::AssetsLibraryWidget(AsynchronousImageCache &imageCache,
         }
     });
 
+     QmlDesignerPlugin::trackWidgetFocusTime(this, Constants::EVENT_ASSETSLIBRARY_TIME);
+
     // init the first load of the QML UI elements
     reloadQmlSource();
 }
@@ -213,26 +201,32 @@ void AssetsLibraryWidget::handleAddAsset()
     addResources({});
 }
 
-void AssetsLibraryWidget::handleExtFilesDrop(const QStringList &simpleFilesPaths,
-                                             const QStringList &complexFilesPaths,
+void AssetsLibraryWidget::handleExtFilesDrop(const QList<QUrl> &simpleFilePaths,
+                                             const QList<QUrl> &complexFilePaths,
                                              const QString &targetDirPath)
 {
-    if (!simpleFilesPaths.isEmpty()) {
+    auto toLocalFile = [](const QUrl &url) { return url.toLocalFile(); };
+
+    QStringList simpleFilePathStrings = Utils::transform<QStringList>(simpleFilePaths, toLocalFile);
+    QStringList complexFilePathStrings = Utils::transform<QStringList>(complexFilePaths,
+                                                                       toLocalFile);
+
+    if (!simpleFilePathStrings.isEmpty()) {
         if (targetDirPath.isEmpty()) {
-            addResources(simpleFilesPaths);
+            addResources(simpleFilePathStrings);
         } else {
-            AddFilesResult result = ModelNodeOperations::addFilesToProject(simpleFilesPaths,
+            AddFilesResult result = ModelNodeOperations::addFilesToProject(simpleFilePathStrings,
                                                                            targetDirPath);
             if (result == AddFilesResult::Failed) {
                 Core::AsynchronousMessageBox::warning(tr("Failed to Add Files"),
                                                       tr("Could not add %1 to project.")
-                                                          .arg(simpleFilesPaths.join(' ')));
+                                                          .arg(simpleFilePathStrings.join(' ')));
             }
         }
     }
 
-    if (!complexFilesPaths.empty())
-        addResources(complexFilesPaths);
+    if (!complexFilePathStrings.empty())
+        addResources(complexFilePathStrings);
 }
 
 QSet<QString> AssetsLibraryWidget::supportedAssetSuffixes(bool complex)
@@ -301,25 +295,25 @@ QPair<QString, QByteArray> AssetsLibraryWidget::getAssetTypeAndData(const QStrin
     if (!suffix.isEmpty()) {
         if (AssetsLibraryModel::supportedImageSuffixes().contains(suffix)) {
             // Data: Image format (suffix)
-            return {"application/vnd.bauhaus.libraryresource.image", suffix.toUtf8()};
+            return {Constants::MIME_TYPE_ASSET_IMAGE, suffix.toUtf8()};
         } else if (AssetsLibraryModel::supportedFontSuffixes().contains(suffix)) {
             // Data: Font family name
             QRawFont font(assetPath, 10);
             QString fontFamily = font.isValid() ? font.familyName() : "";
-            return {"application/vnd.bauhaus.libraryresource.font", fontFamily.toUtf8()};
+            return {Constants::MIME_TYPE_ASSET_FONT, fontFamily.toUtf8()};
         } else if (AssetsLibraryModel::supportedShaderSuffixes().contains(suffix)) {
             // Data: shader type, frament (f) or vertex (v)
-            return {"application/vnd.bauhaus.libraryresource.shader",
+            return {Constants::MIME_TYPE_ASSET_SHADER,
                 AssetsLibraryModel::supportedFragmentShaderSuffixes().contains(suffix) ? "f" : "v"};
         } else if (AssetsLibraryModel::supportedAudioSuffixes().contains(suffix)) {
             // No extra data for sounds
-            return {"application/vnd.bauhaus.libraryresource.sound", {}};
+            return {Constants::MIME_TYPE_ASSET_SOUND, {}};
         } else if (AssetsLibraryModel::supportedVideoSuffixes().contains(suffix)) {
             // No extra data for videos
-            return {"application/vnd.bauhaus.libraryresource.video", {}};
+            return {Constants::MIME_TYPE_ASSET_VIDEO, {}};
         } else if (AssetsLibraryModel::supportedTexture3DSuffixes().contains(suffix)) {
             // Data: Image format (suffix)
-            return {"application/vnd.bauhaus.libraryresource.texture3d", suffix.toUtf8()};
+            return {Constants::MIME_TYPE_ASSET_TEXTURE3D, suffix.toUtf8()};
         }
     }
     return {};
