@@ -36,19 +36,21 @@
 #include "vcsplugin.h"
 
 #include <aggregation/aggregate.h>
-#include <cpptools/cppmodelmanager.h>
 
 #include <coreplugin/find/basetextfind.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/editormanager/editormanager.h>
 
+#include <extensionsystem/invoker.h>
+#include <extensionsystem/pluginmanager.h>
+
 #include <utils/algorithm.h>
 #include <utils/checkablemessagebox.h>
 #include <utils/completingtextedit.h>
 #include <utils/fileutils.h>
 #include <utils/icon.h>
 #include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 #include <utils/temporarydirectory.h>
 #include <utils/theme/theme.h>
 
@@ -78,30 +80,9 @@ enum { debug = 0 };
 enum { wantToolBar = 0 };
 
 // Return true if word is meaningful and can be added to a completion model
-static bool acceptsWordForCompletion(const char *word)
+static bool acceptsWordForCompletion(const QString &word)
 {
-    if (!word)
-        return false;
-    static const std::size_t minWordLength = 7;
-    return std::strlen(word) >= minWordLength;
-}
-
-// Return the class name which function belongs to
-static const char *belongingClassName(const CPlusPlus::Function *function)
-{
-    if (!function)
-        return nullptr;
-
-    if (auto funcName = function->name()) {
-        if (auto qualifiedNameId =  funcName->asQualifiedNameId()) {
-            if (const CPlusPlus::Name *funcBaseName = qualifiedNameId->base()) {
-                if (auto identifier = funcBaseName->identifier())
-                    return identifier->chars();
-            }
-        }
-    }
-
-    return nullptr;
+    return word.size() >= 7;
 }
 
 /*!
@@ -147,22 +128,21 @@ using namespace Utils;
 
 static inline QString submitMessageCheckScript()
 {
-    return VcsPlugin::instance()->settings().submitMessageCheckScript;
+    return VcsPlugin::instance()->settings().submitMessageCheckScript.value();
 }
 
 class VcsBaseSubmitEditorPrivate
 {
 public:
-    VcsBaseSubmitEditorPrivate(const VcsBaseSubmitEditorParameters *parameters,
-                               SubmitEditorWidget *editorWidget,
+    VcsBaseSubmitEditorPrivate(SubmitEditorWidget *editorWidget,
                                VcsBaseSubmitEditor *q);
 
     SubmitEditorWidget *m_widget;
     QToolBar *m_toolWidget = nullptr;
-    const VcsBaseSubmitEditorParameters *m_parameters;
+    VcsBaseSubmitEditorParameters m_parameters;
     QString m_displayName;
-    QString m_checkScriptWorkingDirectory;
-    SubmitEditorFile *m_file;
+    FilePath m_checkScriptWorkingDirectory;
+    SubmitEditorFile m_file;
 
     QPointer<QAction> m_diffAction;
     QPointer<QAction> m_submitAction;
@@ -170,12 +150,9 @@ public:
     NickNameDialog *m_nickNameDialog = nullptr;
 };
 
-VcsBaseSubmitEditorPrivate::VcsBaseSubmitEditorPrivate(const VcsBaseSubmitEditorParameters *parameters,
-                                                       SubmitEditorWidget *editorWidget,
+VcsBaseSubmitEditorPrivate::VcsBaseSubmitEditorPrivate(SubmitEditorWidget *editorWidget,
                                                        VcsBaseSubmitEditor *q) :
-    m_widget(editorWidget),
-    m_parameters(parameters),
-    m_file(new SubmitEditorFile(parameters, q))
+    m_widget(editorWidget), m_file(q)
 {
     auto completer = new QCompleter(q);
     completer->setCaseSensitivity(Qt::CaseSensitive);
@@ -184,15 +161,23 @@ VcsBaseSubmitEditorPrivate::VcsBaseSubmitEditorPrivate(const VcsBaseSubmitEditor
     m_widget->descriptionEdit()->setCompletionLengthThreshold(4);
 }
 
-VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *parameters,
-                                         SubmitEditorWidget *editorWidget) :
-    d(new VcsBaseSubmitEditorPrivate(parameters, editorWidget, this))
+VcsBaseSubmitEditor::VcsBaseSubmitEditor(SubmitEditorWidget *editorWidget)
 {
+    setWidget(editorWidget);
+    d = new VcsBaseSubmitEditorPrivate(editorWidget, this);
+}
+
+void VcsBaseSubmitEditor::setParameters(const VcsBaseSubmitEditorParameters &parameters)
+{
+    d->m_parameters = parameters;
+    d->m_file.setId(parameters.id);
+    d->m_file.setMimeType(QLatin1String(parameters.mimeType));
+
     setWidget(d->m_widget);
-    document()->setPreferredDisplayName(QCoreApplication::translate("VCS", d->m_parameters->displayName));
+    document()->setPreferredDisplayName(QCoreApplication::translate("VCS", d->m_parameters.displayName));
 
     // Message font according to settings
-    CompletingTextEdit *descriptionEdit = editorWidget->descriptionEdit();
+    CompletingTextEdit *descriptionEdit = d->m_widget->descriptionEdit();
     const TextEditor::FontSettings fs = TextEditor::TextEditorSettings::fontSettings();
     const QTextCharFormat tf = fs.toTextCharFormat(TextEditor::C_TEXT);
     descriptionEdit->setFont(tf.font());
@@ -200,13 +185,13 @@ VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *pa
     QPalette pal;
     pal.setColor(QPalette::Base, tf.background().color());
     pal.setColor(QPalette::Text, tf.foreground().color());
-    pal.setColor(QPalette::Foreground, tf.foreground().color());
+    pal.setColor(QPalette::WindowText, tf.foreground().color());
     if (selectionFormat.background().style() != Qt::NoBrush)
         pal.setColor(QPalette::Highlight, selectionFormat.background().color());
     pal.setBrush(QPalette::HighlightedText, selectionFormat.foreground());
     descriptionEdit->setPalette(pal);
 
-    d->m_file->setModified(false);
+    d->m_file.setModified(false);
     // We are always clean to prevent the editor manager from asking to save.
 
     connect(d->m_widget, &SubmitEditorWidget::diffSelected,
@@ -214,32 +199,33 @@ VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *pa
     connect(descriptionEdit, &QTextEdit::textChanged,
             this, &VcsBaseSubmitEditor::fileContentsChanged);
 
-    const CommonVcsSettings settings = VcsPlugin::instance()->settings();
+    const CommonVcsSettings &settings = VcsPlugin::instance()->settings();
     // Add additional context menu settings
-    if (!settings.submitMessageCheckScript.isEmpty() || !settings.nickNameMailMap.isEmpty()) {
+    if (!settings.submitMessageCheckScript.value().isEmpty()
+            || !settings.nickNameMailMap.value().isEmpty()) {
         auto sep = new QAction(this);
         sep->setSeparator(true);
         d->m_widget->addDescriptionEditContextMenuAction(sep);
         // Run check action
-        if (!settings.submitMessageCheckScript.isEmpty()) {
+        if (!settings.submitMessageCheckScript.value().isEmpty()) {
             auto checkAction = new QAction(tr("Check Message"), this);
             connect(checkAction, &QAction::triggered,
                     this, &VcsBaseSubmitEditor::slotCheckSubmitMessage);
             d->m_widget->addDescriptionEditContextMenuAction(checkAction);
         }
         // Insert nick
-        if (!settings.nickNameMailMap.isEmpty()) {
+        if (!settings.nickNameMailMap.value().isEmpty()) {
             auto insertAction = new QAction(tr("Insert Name..."), this);
             connect(insertAction, &QAction::triggered, this, &VcsBaseSubmitEditor::slotInsertNickName);
             d->m_widget->addDescriptionEditContextMenuAction(insertAction);
         }
     }
     // Do we have user fields?
-    if (!settings.nickNameFieldListFile.isEmpty())
-        createUserFields(settings.nickNameFieldListFile);
+    if (!settings.nickNameFieldListFile.value().isEmpty())
+        createUserFields(settings.nickNameFieldListFile.value());
 
     // wrapping. etc
-    slotUpdateEditorSettings(settings);
+    slotUpdateEditorSettings();
     connect(VcsPlugin::instance(), &VcsPlugin::settingsChanged,
             this, &VcsBaseSubmitEditor::slotUpdateEditorSettings);
     connect(Core::EditorManager::instance(), &Core::EditorManager::currentEditorChanged,
@@ -265,10 +251,11 @@ VcsBaseSubmitEditor::~VcsBaseSubmitEditor()
     delete d;
 }
 
-void VcsBaseSubmitEditor::slotUpdateEditorSettings(const CommonVcsSettings &s)
+void VcsBaseSubmitEditor::slotUpdateEditorSettings()
 {
-    setLineWrapWidth(s.lineWrapWidth);
-    setLineWrap(s.lineWrap);
+    const CommonVcsSettings &s = VcsPlugin::instance()->settings();
+    setLineWrapWidth(s.lineWrapWidth.value());
+    setLineWrap(s.lineWrap.value());
 }
 
 // Return a trimmed list of non-empty field texts
@@ -276,7 +263,7 @@ static inline QStringList fieldTexts(const QString &fileContents)
 {
     QStringList rc;
     const QStringList rawFields = fileContents.trimmed().split(QLatin1Char('\n'));
-    foreach (const QString &field, rawFields) {
+    for (const QString &field : rawFields) {
         const QString trimmedField = field.trimmed();
         if (!trimmedField.isEmpty())
             rc.push_back(trimmedField);
@@ -287,8 +274,11 @@ static inline QStringList fieldTexts(const QString &fileContents)
 void VcsBaseSubmitEditor::createUserFields(const QString &fieldConfigFile)
 {
     FileReader reader;
-    if (!reader.fetch(fieldConfigFile, QIODevice::Text, Core::ICore::mainWindow()))
+    if (!reader.fetch(FilePath::fromString(fieldConfigFile),
+                      QIODevice::Text,
+                      Core::ICore::dialogParent())) {
         return;
+    }
     // Parse into fields
     const QStringList fields = fieldTexts(QString::fromUtf8(reader.data()));
     if (fields.empty())
@@ -357,15 +347,15 @@ void VcsBaseSubmitEditor::setLineWrapWidth(int w)
 
 Core::IDocument *VcsBaseSubmitEditor::document() const
 {
-    return d->m_file;
+    return &d->m_file;
 }
 
 QString VcsBaseSubmitEditor::checkScriptWorkingDirectory() const
 {
-    return d->m_checkScriptWorkingDirectory;
+    return d->m_checkScriptWorkingDirectory.toString();
 }
 
-void VcsBaseSubmitEditor::setCheckScriptWorkingDirectory(const QString &s)
+void VcsBaseSubmitEditor::setCheckScriptWorkingDirectory(const FilePath &s)
 {
     d->m_checkScriptWorkingDirectory = s;
 }
@@ -407,6 +397,17 @@ QStringList VcsBaseSubmitEditor::checkedFiles() const
     return d->m_widget->checkedFiles();
 }
 
+static QSet<FilePath> filesFromModel(SubmitFileModel *model)
+{
+    QSet<FilePath> result;
+    result.reserve(model->rowCount());
+    for (int row = 0; row < model->rowCount(); ++row) {
+        result.insert(FilePath::fromString(
+            QFileInfo(model->repositoryRoot(), model->file(row)).absoluteFilePath()));
+    }
+    return result;
+}
+
 void VcsBaseSubmitEditor::setFileModel(SubmitFileModel *model)
 {
     QTC_ASSERT(model, return);
@@ -421,49 +422,21 @@ void VcsBaseSubmitEditor::setFileModel(SubmitFileModel *model)
     if (!selected.isEmpty())
         d->m_widget->setSelectedRows(selected);
 
-    QSet<QString> uniqueSymbols;
-    const CPlusPlus::Snapshot cppSnapShot = CppTools::CppModelManager::instance()->snapshot();
-
-    // Iterate over the files and get interesting symbols
-    for (int row = 0; row < model->rowCount(); ++row) {
-        const QFileInfo fileInfo(model->repositoryRoot(), model->file(row));
-
-        // Add file name
-        uniqueSymbols.insert(fileInfo.fileName());
-
-        const QString filePath = fileInfo.absoluteFilePath();
-        // Add symbols from the C++ code model
-        const CPlusPlus::Document::Ptr doc = cppSnapShot.document(filePath);
-        if (!doc.isNull() && doc->control()) {
-            const CPlusPlus::Control *ctrl = doc->control();
-            CPlusPlus::Symbol **symPtr = ctrl->firstSymbol(); // Read-only
-            while (symPtr != ctrl->lastSymbol()) {
-                const CPlusPlus::Symbol *sym = *symPtr;
-
-                const CPlusPlus::Identifier *symId = sym->identifier();
-                // Add any class, function or namespace identifiers
-                if ((sym->isClass() || sym->isFunction() || sym->isNamespace())
-                        && (symId && acceptsWordForCompletion(symId->chars())))
-                {
-                    uniqueSymbols.insert(QString::fromUtf8(symId->chars()));
-                }
-
-                // Handle specific case : get "Foo" in "void Foo::function() {}"
-                if (sym->isFunction() && !sym->asFunction()->isDeclaration()) {
-                    const char *className = belongingClassName(sym->asFunction());
-                    if (acceptsWordForCompletion(className))
-                        uniqueSymbols.insert(QString::fromUtf8(className));
-                }
-
-                ++symPtr;
-            }
-        }
+    const QSet<FilePath> files = filesFromModel(model);
+    // add file names to completion
+    QSet<QString> completionItems = Utils::transform(files, &FilePath::fileName);
+    QObject *cppModelManager = ExtensionSystem::PluginManager::getObjectByName("CppModelManager");
+    if (cppModelManager) {
+        const auto symbols = ExtensionSystem::invoke<QSet<QString>>(cppModelManager,
+                                                                    "symbolsInFiles",
+                                                                    files);
+        completionItems += Utils::filtered(symbols, acceptsWordForCompletion);
     }
 
     // Populate completer with symbols
-    if (!uniqueSymbols.isEmpty()) {
+    if (!completionItems.isEmpty()) {
         QCompleter *completer = d->m_widget->descriptionEdit()->completer();
-        QStringList symbolsList = Utils::toList(uniqueSymbols);
+        QStringList symbolsList = Utils::toList(completionItems);
         symbolsList.sort();
         completer->setModel(new QStringListModel(symbolsList, completer));
     }
@@ -489,7 +462,7 @@ QStringList VcsBaseSubmitEditor::rowsToFiles(const QList<int> &rows) const
 
 void VcsBaseSubmitEditor::slotDiffSelectedVcsFiles(const QList<int> &rawList)
 {
-    if (d->m_parameters->diffType == VcsBaseSubmitEditorParameters::DiffRows)
+    if (d->m_parameters.diffType == VcsBaseSubmitEditorParameters::DiffRows)
         emit diffSelectedRows(rawList);
     else
         emit diffSelectedFiles(rowsToFiles(rawList));
@@ -545,13 +518,14 @@ static QString withUnusedMnemonic(QString string, const QList<QPushButton *> &ot
 }
 
 VcsBaseSubmitEditor::PromptSubmitResult
-        VcsBaseSubmitEditor::promptSubmit(VcsBasePlugin *plugin,
-                                          bool *promptSetting,
+        VcsBaseSubmitEditor::promptSubmit(VcsBasePluginPrivate *plugin,
+                                          bool *promptSettingOld,
                                           bool forcePrompt,
-                                          bool canCommitOnFailure)
+                                          bool canCommitOnFailure,
+                                          BoolAspect *promptSetting)
 {
-    bool dummySetting = false;
-    if (!promptSetting)
+    BoolAspect dummySetting;
+    if (!promptSetting && !promptSettingOld)
         promptSetting = &dummySetting;
     auto submitWidget = static_cast<SubmitEditorWidget *>(this->widget());
 
@@ -562,7 +536,8 @@ VcsBaseSubmitEditor::PromptSubmitResult
 
     QString errorMessage;
 
-    const bool prompt = forcePrompt || *promptSetting;
+    const bool value = promptSettingOld ? *promptSettingOld : promptSetting->value();
+    const bool prompt = forcePrompt || value;
 
     // Pop up a message depending on whether the check succeeded and the
     // user wants to be prompted
@@ -571,8 +546,7 @@ VcsBaseSubmitEditor::PromptSubmitResult
         return SubmitConfirmed;
     CheckableMessageBox mb(Core::ICore::dialogParent());
     const QString commitName = plugin->commitDisplayName();
-    mb.setWindowTitle(tr("Close %1 %2 Editor")
-                      .arg(plugin->versionControl()->displayName(), commitName));
+    mb.setWindowTitle(tr("Close %1 %2 Editor").arg(plugin->displayName(), commitName));
     mb.setIcon(QMessageBox::Question);
     QString message;
     if (canCommit) {
@@ -585,9 +559,9 @@ VcsBaseSubmitEditor::PromptSubmitResult
     }
     mb.setText(message);
     mb.setCheckBoxText(tr("Prompt to %1").arg(commitName.toLower()));
-    mb.setChecked(*promptSetting);
+    mb.setChecked(value);
     // Provide check box to turn off prompt ONLY if it was not forced
-    mb.setCheckBoxVisible(*promptSetting && !forcePrompt);
+    mb.setCheckBoxVisible(value && !forcePrompt);
     QDialogButtonBox::StandardButtons buttons = QDialogButtonBox::Close | QDialogButtonBox::Cancel;
     if (canCommit || canCommitOnFailure)
         buttons |= QDialogButtonBox::Ok;
@@ -603,8 +577,12 @@ VcsBaseSubmitEditor::PromptSubmitResult
         commitButton->setText(withUnusedMnemonic(commitName,
                               {cancelButton, mb.button(QDialogButtonBox::Close)}));
     }
-    if (mb.exec() == QDialog::Accepted)
-        *promptSetting = mb.isChecked();
+    if (mb.exec() == QDialog::Accepted) {
+        if (promptSettingOld)
+            *promptSettingOld = mb.isChecked();
+        else
+            promptSetting->setValue(mb.isChecked());
+    }
     QAbstractButton *chosen = mb.clickedButton();
     if (!chosen || chosen == cancelButton)
         return SubmitCanceled;
@@ -660,13 +638,13 @@ bool VcsBaseSubmitEditor::checkSubmitMessage(QString *errorMessage) const
     return rc;
 }
 
-static inline QString msgCheckScript(const QString &workingDir, const QString &cmd)
+static QString msgCheckScript(const FilePath &workingDir, const QString &cmd)
 {
     const QString nativeCmd = QDir::toNativeSeparators(cmd);
     return workingDir.isEmpty() ?
            VcsBaseSubmitEditor::tr("Executing %1").arg(nativeCmd) :
            VcsBaseSubmitEditor::tr("Executing [%1] %2").
-           arg(QDir::toNativeSeparators(workingDir), nativeCmd);
+           arg(workingDir.toUserOutput(), nativeCmd);
 }
 
 bool VcsBaseSubmitEditor::runSubmitMessageCheckScript(const QString &checkScript, QString *errorMessage) const
@@ -679,19 +657,18 @@ bool VcsBaseSubmitEditor::runSubmitMessageCheckScript(const QString &checkScript
     // Run check process
     VcsOutputWindow::appendShellCommandLine(msgCheckScript(d->m_checkScriptWorkingDirectory,
                                                            checkScript));
-    QProcess checkProcess;
+    QtcProcess checkProcess;
     if (!d->m_checkScriptWorkingDirectory.isEmpty())
         checkProcess.setWorkingDirectory(d->m_checkScriptWorkingDirectory);
-    checkProcess.start(checkScript, QStringList(saver.fileName()));
-    checkProcess.closeWriteChannel();
+    checkProcess.setCommand({FilePath::fromString(checkScript), {saver.filePath().toString()}});
+    checkProcess.start();
     if (!checkProcess.waitForStarted()) {
         *errorMessage = tr("The check script \"%1\" could not be started: %2").arg(checkScript, checkProcess.errorString());
         return false;
     }
     QByteArray stdOutData;
     QByteArray stdErrData;
-    if (!SynchronousProcess::readDataFromProcess(checkProcess, 30, &stdOutData, &stdErrData, false)) {
-        SynchronousProcess::stopProcess(checkProcess);
+    if (!checkProcess.readDataFromProcess(30, &stdOutData, &stdErrData, false)) {
         *errorMessage = tr("The check script \"%1\" timed out.").
                         arg(QDir::toNativeSeparators(checkScript));
         return false;
